//			   GFX, graphical extension for Turbo Vision.
//					   Written By: Simor Balazs
//								V 2.10
//
//	  This software can be freely distributed provided, that the author
//	 message remains intact, and it is not used for professional purposes.
// If you make modifications to the software and you want to redistribute it,
//	   please let the user know, that it is not the original version.
//   	  You may NOT redistribute the source code of the software, it is
//	            			not shareware


//This headerfile contains definitions for use with the GFX graphics
//                         interface for TV

#ifndef __GFX_H_
#define __GFX_H_

#define Uses_TApplication
#define Uses_TScreen
#include <tv.h>
#include <graphics.h>

//--------------------------External variables------------------------

extern int xcharsize,ycharsize; //placed in Mytmouse
extern Boolean useAltWriteMethod;
extern unsigned short far *Cache;
extern int maxx, maxy;

//------------------------Global functions----------------------------
extern int GFXinitialize(int gdriver, int gmode);
extern void GFXclose(void);
extern void  ClearCacheRegion(const TRect& r, Boolean outline);


//----------------------------Classes---------------------------------



class GRect : public TRect
{
public:
	GRect(void) {}
	GRect(TRect base);
	GRect(TPoint topleft, TPoint bottomright)
		{ a.x = topleft.x;
		  a.y = topleft.y;
		  b.x = bottomright.x;
		  b.y = bottomright.y;
		}
	GRect(int ax, int ay, int bx, int by)
		{
		a.x = ax;
		a.y = ay;
		b.x = bx;
		b.y = by;
		}
	int distance(TPoint &p);
};


struct Win
{
	GRect w;
	int XCorr, YCorr; //Correction values for this window
	Win *next;
};


class GView : public TView
{
public:
	GView (const TRect& bounds) : TView (bounds) {winList = 0;} 

	virtual void draw();
	virtual void paint() {TView::draw();}
	int beginPaint(void);
	void endPaint(void);
	GRect getGClipRect() { return GRect( TView::getClipRect() ); }
	GRect getGBounds() { return GRect( TView::getBounds() ); }
	GRect getGExtent() { return GRect( TView::getExtent() ); }
	GRect getGAbsBounds();
	TRect getAbsBounds();
	Boolean isClear();
	void makeArea(char *area);
	void freeWinList(void);
	void makeWinList(char *area, TRect& rect, TPoint& orig);
	void imagedimensions(void far *buf, unsigned *width, unsigned *height);

	void arc(int x, int y, int stangle, int endangle, int radius);
	void bar(int left, int top, int right, int bottom);
	void bar3d(int left, int top, int right, int bottom, int depth,
				int topflag);
	void circle(int x, int y, int radius);
	void cleardevice(void) {}
	void clearviewport(void);
	void drawpoly(int numpoints, int far *polypoints);
	void ellipse(int x, int y, int stangle, int endangle,
				int xradius, int yradius);
	void fillellipse(int x, int y, int xradius, int yradius);
	void fillpoly(int numpoints, int far *polypoints);
	#pragma argsused
	void floodfill(int x, int y, int border) {}
	void getimage(int left, int top, int right, int bottom,
				void far *bitmap);
	#pragma argsused
	void getpixel(int x, int y) {}
	int  getx(void);
	int  gety(void);
	unsigned long imagesize(int left, int top, int right, int bottom);
	void line(int x1, int y1, int x2, int y2);
	void linerel(int dx, int dy);
	void lineto(int x, int y);
	void moverel(int dx, int dy);
	void moveto(int x, int y);
	void outtext(char far *textstring);
	void outtextxy(int x, int y, char far *textstring);
	void pieslice(int x, int y, int stangle, int endangle, int radius);
	void putimage(int left, int top, void far *bitmap, int op);
	void putpixel(int x, int y, int color);
	void rectangle(int left, int top, int right, int bottom);
	void sector(int x, int y, int stangle, int endangle,
			int xradius, int yradius);
	#pragma argsused
	void setactivepage(int page) {}
	#pragma argsused
	void setviewport(int left, int top, int right, int bottom, int clip) {}
	#pragma argsused
	void setvisualpage(int page) {}
private:

	TPoint myCP; //CP (Current Point) in the GView see graphics.h doc

	Win *winList; //actual winlist (valid after a call to beginPaint until
				//calling endPaint)
	TRect crect; //for internal use in beginPaint and endPaint
}; //GView



enum videoModes //this list is extended with graphics modes
	{
	smBW80      = 0x0002,
	smCO80      = 0x0003,
	smMono      = 0x0007,
	smFont8x8   = 0x0100,
	smEGALO     = 0x1001,
	smEGAHI     = 0x1002,
	smVGALO		= 0x1003,
	smVGAMED	= 0x1004,
	smVGAHI		= 0x1005,
	smDETECT    = 0x2000
	};


class GProgram : public TProgram
{
public:
	GProgram() : TProgInit(
			&GProgram::initStatusLine,
			&GProgram::initMenuBar,
			&GProgram::initDeskTop )  { setScreenMode(smDETECT);}
	GProgram(ushort mode) : TProgInit(
			&GProgram::initStatusLine,
			&GProgram::initMenuBar,
			&GProgram::initDeskTop )  { setScreenMode(mode);}
	~GProgram() { TDisplay::setCrtMode(smCO80);}

	void setScreenMode(ushort mode); //overrriding original
};

class GApplication : public TApplication
{
public:
	GApplication() : TProgInit(
			&GApplication::initStatusLine,
			&GApplication::initMenuBar,
			&GApplication::initDeskTop )  {setScreenMode(smDETECT);}
	GApplication(ushort mode) : TProgInit(
			&GApplication::initStatusLine,
			&GApplication::initMenuBar,
			&GApplication::initDeskTop )  {setScreenMode(mode);}
	~GApplication() { TDisplay::setCrtMode(smCO80);}

	void setScreenMode(ushort mode); //overrriding original
};


#endif