//			   GFX, graphical extension for Turbo Vision.
//					   Written By: Simor Balazs
//								V 2.10
//
//	  This software can be freely distributed provided, that the author
//	 message remains intact, and it is not used for professional purposes.
// If you make modifications to the software and you want to redistribute it,
//	   please let the user know, that it is not the original version.
//   	  You may NOT redistribute the source code of the software, it is
//	            			not shareware

#define Uses_TApplication
#define Uses_TDeskTop
#define Uses_TMenuBar
#define Uses_TSubMenu
#define Uses_TMenuItem
#define Uses_TKeys
#define Uses_TScreen
#define Uses_TMouse
#define Uses_MsgBox
#define Uses_TWindow
#define Uses_TDialog
#define Uses_TButton
#define Uses_TInputLine
#define Uses_fpstream
#define Uses_TStaticText
#include <tv.h>

#include <graphics.h>
#include "gfx.h"

#include <timer.h>
#include <alloc.h>

//undefine this symbol, if you want to see the speed difference between the
//cached and non-cached graphics view at "File/Pop Up Graphics" menu.
#define Graphics_caching

const GRFX = 120;
const toGR = 121;
const toTX = 122;
const About= 123;
const MVMNT = 124;
const BarCh = 125;
const TxtTst = 126;
const PieCh = 127;


class IdleObject
{
public:
	virtual void idle_function(void) {}

	IdleObject *n; //pointer to next IdleObject
};

class Movie : public GView, public IdleObject
{
	int states;
	long sz;
	char *pict;
	int state;
public:
	Movie(const TRect& bounds);
	~Movie(void);
	virtual void idle_function(void);
	virtual void paint(void);
};

class MyMoving : public GView, public IdleObject
{
public:
	MyMoving(const TRect& bounds);
	~MyMoving(void);
	virtual void idle_function(void);
	void paint(void);

private:
	int state;
	int max;
	TPoint cntr;
};

class MyDrawing : public GView
{
#ifdef Graphics_caching
	Boolean stored;
	void *img;
	GRect ex; //extent, for  which the img is stored
#endif
public:
	MyDrawing (const TRect& bounds) : GView(bounds) {
#ifdef Graphics_caching
	stored = False;
	img = 0;
#endif
	}
#ifdef Graphics_caching
	~MyDrawing () { if (img) farfree(img); }
#endif
	virtual void paint(void);
};

class BarChart : public GView
{
public:
	BarChart (const TRect& bounds) : GView(bounds) {}
	virtual void paint(void);
};

class TextTst : public GView
{
public:
	TextTst (const TRect& bounds) : GView(bounds) {}
	virtual void paint(void);
};

class PieChart : public GView
{
public:
	PieChart (const TRect& bounds) : GView(bounds) {}
	virtual void paint(void);
};

class TMyApp : public GApplication
	{
	public:
		TMyApp();

		virtual void handleEvent(TEvent &event);
		static TMenuBar *initMenuBar( TRect r );
		virtual void idle(void);
		int registeridle(IdleObject *obj);
		void unregisteridle(IdleObject *obj);

	private:
	IdleObject *idleobj;

	Timer idle_timer;
	};


TMyApp *app;



Movie::Movie(const TRect& bounds) : GView(bounds)
{
	sz = 3963;
	states = 4;
	pict = (char *)farmalloc(sz*states);
	state=0;

fpstream in;	//reading the file containing the silly head.
	in.open("images",ios::binary|ios::in|ios::nocreate);
	if (in.fail())
		{
		free(pict);
		pict = 0;
		return;
		}
	in.readBytes(pict, sz*states);
	in.close();

unsigned w,h;
	if (TScreen::screenMode>0x1000)  //only in graphics modes
		{
		imagedimensions(pict, &w, &h);
		growTo(w/xcharsize, h/ycharsize+1); //growing to the necessary size
		}
}

Movie::~Movie(void)
{
	app->unregisteridle(this);
	if (pict) free(pict);
}

void Movie::paint(void)
{
	TView::draw(); //original background
	if (!pict) return;
	putimage(0,0,pict+sz*state,COPY_PUT);
}

void Movie::idle_function(void)
{
	if (!pict) return;
	if (1 != beginPaint()) return;
	state = (state+1)%states;
	putimage(0,0,pict+sz*state,COPY_PUT);
	endPaint();
}


MyMoving::~MyMoving(void)
{
	app->unregisteridle(this);
}

MyMoving::MyMoving(const TRect& bounds) : GView(bounds)
{
GRect ext;

	ext = getGExtent();

	cntr.x = ext.b.x/2;
	cntr.y = ext.b.y/2;

	max = (cntr.x > cntr.y ? cntr.x : cntr.y);
	state = 0;
}

void MyMoving::paint(void)
{
	clearviewport();
}

void MyMoving::idle_function(void)
{
	if (1 != beginPaint()) return;
	setcolor(BLACK);
	circle (cntr.x , cntr.y , state);

	GRect ext = getGExtent(); //New part ensures that the circles will be in
	cntr.x = ext.b.x/2; //the center even after bound changes
	cntr.y = ext.b.y/2;
	max = (cntr.x > cntr.y ? cntr.x : cntr.y);

	state = max < state+4 ? 0 : state + 4;

	setcolor(YELLOW);
	circle (cntr.x , cntr.y , state);
	endPaint();
}

void MyDrawing::paint(void)
{
	GRect ext = getGExtent();
#ifdef Graphics_caching
	if (stored && ext == ex)
		{
		putimage (0, 0, img, COPY_PUT);
		return;
		} //if
#endif
	TPoint cntr;   //Centerpoint
	cntr.x = ext.b.x/2;
	cntr.y = ext.b.y/2;
	int max = (cntr.x > cntr.y ? cntr.x : cntr.y);

	clearviewport();
	//TView::draw();

	for (int i = max; i>0; i-=4)
		{
		setcolor ( (i/4)%16);
		setfillstyle (SOLID_FILL, (i/4)%16);
		fillellipse (cntr.x , cntr.y , i,i);
		} //for

#ifdef Graphics_caching
	if (!isClear()) return;

	if (stored)
		farfree(img);

	ex = ext;

	GRect a = getGAbsBounds();

	img = farmalloc(imagesize(a.a.x, a.a.y, a.b.x, a.b.y) );

	//::setviewport(0,0,getmaxx(),getmaxy(),1);
	getimage(a.a.x, a.a.y, a.b.x, a.b.y, img);

	stored = True;
#endif
}

void BarChart::paint(void)
{
	clearviewport();
	GRect ext = getGExtent();

	setcolor(WHITE);
	for (int i=0; i<4; i++)
		{
		setfillstyle(SOLID_FILL, i+9);
		bar3d(i*ext.b.x/4+5, 10+i*5, (i+1)*ext.b.x/4-5, ext.b.y, 10, 1);
		} //for
}

void TextTst::paint()
{
	GRect ext = getGExtent();
	clearviewport();
	setcolor(WHITE);
	settextstyle (GOTHIC_FONT, HORIZ_DIR,6);
	settextjustify(CENTER_TEXT, CENTER_TEXT);
	outtextxy(ext.b.x/2, ext.b.y/2, "A Test");
}

void PieChart::paint()
{
int data[5]={13, 32, 100, 54, 90};
int xas, yas;

	GRect ext = getGExtent();
	getaspectratio(&xas, &yas);
	int r = ((long)ext.b.y*yas>(long)ext.b.x*xas ?
			(long)ext.b.x*xas/yas : (long)ext.b.y*yas/xas) *7/8/2;
	clearviewport();
	setcolor(WHITE);
	int s = 0;
	for (int i=0; i<5; i++)
		{
		setfillstyle(SOLID_FILL, 9+i);
		pieslice(ext.b.x/2, ext.b.y/2, s, s+data[i], r);
		s += data[i];
		} //for

	setfillstyle(SOLID_FILL, 9+i);
	pieslice(ext.b.x/2+10, ext.b.y/2 + 5, s, 359, r);
}

void TMyApp::idle(void)
{
	TProgram::idle();

	idle_timer.stop();
	double t = idle_timer.time();
	if (t < .05)
		{
		idle_timer.start();
		return;
		} //if
	idle_timer.reset();
	idle_timer.start();

IdleObject *p=idleobj;

	while (p)
		{
		p->idle_function();
		p = p->n;
		} //while
}


int TMyApp::registeridle( IdleObject *obj )
{
const Number_of_idle_objects = 5;
IdleObject *p0 = 0;
IdleObject *p = idleobj;
int i = 1;

	while (p && i<Number_of_idle_objects)
		{
		p0 = p;
		p = p->n;
		i++;
		}//while
	if (p) return 0; // too many idle objects
	if (p0)
		p0->n=obj;// adding to queue
	else
		idleobj = obj;
	obj->n = 0; //closing queue
	return 1;
}

void TMyApp::unregisteridle(IdleObject *obj)
{
IdleObject *p = idleobj;
IdleObject *p0;

	if (idleobj == obj)
		{
		idleobj = idleobj->n; //unlinking the first
		return;
		} //if

	while(p)
		{
		p0 = p;
		p = p->n;
		if(p == obj)
			{
			p0->n = p->n; //unlinking p
			return;
			} //if
		} //while
}

void TMyApp::handleEvent (TEvent &event)
{
	switch (event.message.command)
		{
		case GRFX:
			TDialog *w = new TDialog(TRect(0,0, 50, 11), "Graphics");
			w->options &= ~ofBuffered;
			w->insert (new TButton( TRect (38, 8, 48, 10), "~O~K", cmOK, bfDefault));
			w->insert (new TInputLine(TRect (2, 8, 36,9),32) );
			w->insert (new MyDrawing(TRect(2, 2, 24, 6) ));
			w->insert (new MyDrawing(TRect(26, 2, 48, 6) ));
			deskTop->insert(w);
			clearEvent(event);
		break;

		case BarCh:
			w = new TDialog(TRect(0,0, 50, 11), "Bar Chart");
			w->options &= ~ofBuffered;
			w->insert (new TButton( TRect (38, 8, 48, 10), "~O~K", cmOK, bfDefault));
			w->insert (new TInputLine(TRect (2, 8, 36,9),32) );
			w->insert (new BarChart(TRect(2, 2, 24, 6) ));
			w->insert (new BarChart(TRect(26, 2, 48, 6) ));
			deskTop->insert(w);
			clearEvent(event);
		break;

		case TxtTst:
			w = new TDialog(TRect(0,0, 50, 11), "Text Test");
			w->options &= ~ofBuffered;
			w->insert (new TButton( TRect (38, 8, 48, 10), "~O~K", cmOK, bfDefault));
			w->insert (new TInputLine(TRect (2, 8, 36,9),32) );
			w->insert (new TextTst(TRect(2, 2, 24, 6) ));
			w->insert (new TextTst(TRect(26, 2, 48, 6) ));
			deskTop->insert(w);
			clearEvent(event);
		break;

		case PieCh:
			w = new TDialog(TRect(0,0, 35, 11), "Pie Chart");
			w->options &= ~ofBuffered;
			w->flags |= wfGrow;
			TView *pch = new PieChart( TRect (1, 1, 34, 10));
			pch->growMode = gfGrowHiX | gfGrowHiY; //moves with its owner.
			w->insert (pch);
			deskTop->insert(w);
			clearEvent(event);
		break;

		case MVMNT:
			TDialog *m = new TDialog(TRect(0,0, 50, 11), "Moving Graphics");
			m->options &= ~ofBuffered;
			m->flags |= wfGrow;
			MyMoving *p = new MyMoving(TRect(1, 1, 49, 10) );
			p->growMode = gfGrowHiX | gfGrowHiY;
			m->insert (p);
			if (registeridle(p))
				deskTop->insert(m);
			else
				{
				messageBox("\x03 Sorry, cannot insert more Moving Graphics.",
					mfError | mfOKButton);
				destroy (m);
				}
			clearEvent(event);
		break;

		case toTX:
			app->disableCommand(toTX);
			app->enableCommand(toGR);

			app->setScreenMode(smCO80);
		break;

		case toGR:
			app->disableCommand(toGR);
			app->enableCommand(toTX);

			app->setScreenMode(smVGAHI);
		break;

		case About:
			w = new TDialog(TRect(20,5, 60, 14), "About");
			w->insert (new TButton( TRect (7, 6, 17, 8), "~O~K", cmOK, bfDefault));
			w->insert (new TStaticText(TRect(2,2,26,5),
					"\x03TV in Graphics mode\n"
					"\x03Written By Simor Balazs\n"
					"\x03Source is (C) 1993, 1994") );
			Movie *mv= new Movie(TRect(27,1,28,2));
			if (!registeridle(mv))
				destroy(mv);
			else
				w->insert(mv);

			deskTop->execView(w);
			destroy (w);

		break;
		} //switch

	TApplication::handleEvent(event);
}//handleEvent

TMyApp::TMyApp() :
		TProgInit( &TMyApp::initStatusLine, &TMyApp::initMenuBar,
				&TMyApp::initDeskTop ) ,
		GApplication(smVGAMED)
	{
	idleobj = 0;
	idle_timer.reset();
    idle_timer.start();
	}

TMenuBar *TMyApp::initMenuBar( TRect r )
	{
	r.b.y = r.a.y+1;

	return new TMenuBar( r,
		*new TSubMenu( "~F~ile", kbNoKey )+
			*new TMenuItem( "Pop up graphics", GRFX, kbNoKey, hcNoContext,0) +
			*new TMenuItem( "Pop up movement", MVMNT, kbNoKey, hcNoContext,0) +
			*new TMenuItem( "Pop up bar chart", BarCh, kbNoKey, hcNoContext,0) +
			*new TMenuItem( "Pop up pie chart", PieCh, kbNoKey, hcNoContext,0) +
			*new TMenuItem( "Pop up text test", TxtTst, kbNoKey, hcNoContext,0) +
			newLine() +
			*new TMenuItem( "Graphics mode", toGR, kbNoKey, hcNoContext,0) +
			*new TMenuItem( "Text mode", toTX, kbNoKey, hcNoContext,0) +
			newLine() +
			*new TMenuItem( "E~x~it", cmQuit, kbAltX, hcNoContext, 0 ) +
		*new TSubMenu( "~H~elp", kbNoKey )+
			*new TMenuItem( "~A~bout", About, kbNoKey)
		);
	}


int main()
	{
	app = new(TMyApp);
	app->disableCommand(toGR);
	app->run();
	delete app;
	return 0;
	}
